#include "Model.h"
#include "SamplingBehavior.h"

Angle_Eta::Angle_Eta(string system_filename, double beta, double K, string output_filepath) : Model(output_filepath){
	this->beta = beta;
	this->K = K;

	load_network(system_filename);
	create_graph();
	reference_generators = {0};
} 

void Angle_Eta::update() {
	anglesampler->sweep(*this);
	etasampler->sweep(*this);
}

void Angle_Eta::print(ofstream &output_stream) {
	output_stream << energy() << "\t";
	for (const auto &val : angles) output_stream << val << "\t";
	for (const auto &val : etas) output_stream << val << "\t";
	output_stream << "\n";
}

double Angle_Eta::energy() {
	double output = 0;
	double d_ij;
	int index;

	for (int i = 0; i < n_g; i++)
	{
		for (int j = n_g; j < n_g+n; j++)
		{
			d_ij = angles[i] - angles[j];

			output -= B[i][j] * (1 - cos(d_ij));
		}
	}

	for (int i = n_g; i < n_g+n; i++)
	{
		for (int j = i+1; j < n_g+n; j++)
		{
			index = line_index[i-n_g][j-n_g];
			if (index != 0)
			{
				d_ij = angles[i] - angles[j];

				output -= B[i][j] * (1 - cos(d_ij)) * etas[index-1];
			}
		}
	}

	for (int i = 1; i < n_g+n; i++)
	{
		output += P[i] * angles[i];
	}

	for (int i = 0; i < n_l; i++)
	{
		output -= K * W[i] * (big_F(etas[i]) + (1.01 - etas[i]) * big_F(1 - etas[i]));
	}

	return output;
}

void Angle_Eta::load_network(string system_filename) {
	namespace pt = property_tree;

	// Create a root for the JSON file structure
	pt::ptree root;

	// Load in the json file
	pt::read_json(system_filename, root);

	// Read the system data
	this->n_g = root.get<int>("n_g");
	this->n = root.get<int>("n");
	this->n_l = root.get<int>("n_l");

	this->line_index = read_int_matrix(root, "line_index");
	this->index_to_lines = read_int_matrix(root, "index_to_lines");
	this->W = read_vector(root, "W");

	this->B = read_double_matrix(root, "B");
	this->P = read_vector(root, "P");
	this->base_P = P;

	vector<double> initial_condition = read_vector(root, "initial_condition");
	angles = vector<double>(initial_condition.begin(), initial_condition.begin() + n_g + n);
	etas = vector<double>(initial_condition.end()-n_l, initial_condition.end());
}