#include "Model.h"
#include "SamplingBehavior.h"

Eta_Model::Eta_Model(string system_filename, double lambda, string output_filepath) : Model(output_filepath){
	this->lambda = lambda;

	load_network(system_filename);
	// create_graph();
} 

void Eta_Model::update() {
	etasampler->sweep(*this);
}

void Eta_Model::print(ofstream &output_stream) {
	output_stream << energy() << "\t";
	for (const auto &val : etas) output_stream << val << "\t";
	output_stream << "\n";
}

double Eta_Model::energy() {
	double output = 0;

	for (int i = 0; i < n_l; i++)
	{
		output += W[i] * (lambda * etas[i] - big_F(etas[i]));
		// output += (lambda * etas[i] - big_F(etas[i]));
	}

	return output;
}

void Eta_Model::load_network(string system_filename) {
	namespace pt = property_tree;

	// Create a root for the JSON file structure
	pt::ptree root;

	// Load in the json file
	pt::read_json(system_filename, root);

	// Read the system data
	this->n_g = root.get<int>("n_g");
	this->n = root.get<int>("n");
	this->n_l = root.get<int>("n_l");

	// this->line_index = read_int_matrix(root, "line_index");
	// this->index_to_lines = read_int_matrix(root, "index_to_lines");
	this->W = read_vector(root, "W");

	this->B = read_double_matrix(root, "B");
	// this->P = read_vector(root, "P");
	// this->base_P = P;

	vector<double> initial_condition = read_vector(root, "initial_condition");
	etas = vector<double>(initial_condition.end()-n_l, initial_condition.end());
}