#ifndef MODEL_H
#define MODEL_H

#include <vector>
#include <iostream>
#include <boost/config.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/connected_components.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>

using namespace std;
using namespace boost;

typedef adjacency_list<vecS, vecS, undirectedS> Graph;

class SamplingBehavior;

class Model {
	public:
		Model(string output_filepath);
		~Model();

		virtual void update() = 0;
		virtual void print(ofstream &output_stream) = 0;

		virtual double energy() = 0;
		double big_F(double eta);

		// General graph properties
		vector<vector<int>> line_index;
		vector<vector<int>> index_to_lines;
		vector<vector<double>> B;
		Graph G;
		int n_components;
		int n_g, n, n_l;

		string output_filepath;

	protected:
		void create_graph();
		vector<double> read_vector(property_tree::ptree root, string key);
		vector<vector<int>> read_int_matrix(property_tree::ptree root, string key);
		vector<vector<double>> read_double_matrix(property_tree::ptree root, string key);

};

// Eta model derives from model, Angle_Eta model derives from Eta model

// class Eta_Model: public Model {
// 	public:
// 		Eta_Model(string system_filename, double lambda, string output_filepath);
// 		void update();
// 		void print(ofstream &output_stream);
// 		double energy();
		
// 		void load_network(string system_filename);

// 		vector<double> etas, W;

// 		double lambda;

// 		SamplingBehavior* etasampler;
// };

class Angle_Eta: public Model {
	public:
		Angle_Eta(string system_filename, double beta, double K, string output_filepath); //Remeber to set reference generators!!!
		void update();
		void print(ofstream &output_stream);
		double energy();

		void load_network(string system_filename);
		
		vector<double> angles, etas, base_P, P, W;
		vector<int> reference_generators;

		double beta, K;

		SamplingBehavior* anglesampler;
		SamplingBehavior* etasampler;
};

#endif