#include "Montecarlo.h"
#include "SamplingBehavior.h"
#include "Model.h"
#include <cstdlib>
#include <string>
#include <thread>
#include <mutex>

mutex mu;

void run_angle_eta_simulation(int n_steps, double beta, double K, string system_filename, string output_folder, string sampling) {
	for (int j = 0; j < 1; j++) {
		// Create filepath for the output
		string first = system_filename.substr(0,system_filename.size()-5); // Remove .json extension
		string output_filepath = "./data/" + output_folder + "/" + first + "beta=" + to_string(beta) + "K=" + to_string(K) + "j=" + to_string(j) + ".dat";

		// Instantiate model and inject the chosen sampling behavior
		Angle_Eta model(system_filename, beta, K, output_filepath);
		AngleIntervalSampling anglesampler(-0.5235, 0.5235);
		model.anglesampler = &anglesampler;

		if (sampling == "interval") {
			EtaIntervalSampling etasampler(-1./200, 1./200);
			model.etasampler = &etasampler;
		} else {
			EtaGlobalSampling etasampler(0.0, 1.0);			
			model.etasampler = &etasampler;
		}

		// Start the simulation
		Montecarlo simulation(model, n_steps, output_filepath);
		simulation.run();

		// Threadsafe printing
		mu.lock();
		cout << "K = " << K << " beta = " << beta << " j = " << j << " E = " << model.energy() << endl;
		mu.unlock();
	}
}

// void run_eta_simulation(int n_steps, double lambda, string system_filename, string output_folder) {
// 	for (int j = 0; j < 2; j++) {
// 		string first = system_filename.substr(0,system_filename.size()-5); // Remove .json extension
// 		string output_filepath = "./data/" + output_folder + "/" + first + "lambda=" + to_string(lambda) + "j=" + to_string(j) + ".dat";

// 		Eta_Model model(system_filename, lambda, output_filepath);

// 		EtaIntervalSampling etasampler(-1./200, 1./200);
// 		model.etasampler = &etasampler;

// 		Montecarlo simulation(model, n_steps, output_filepath);
// 		simulation.run();

// 		mu.lock();
// 		cout << "lambda = " << lambda << " j = " << j << " E = " << model.energy() << endl;
// 		mu.unlock();
// 	}
// }

// Here we can optimize creation of threads to be limited to std::thread::hardware_concurrency().
// Look into Semaphore!!
void threaded_sweep(int n_steps, string system_filename, string output_folder, string sampling, string sweeping) {
	vector<double> beta = {0.1, 0.5, 1.0, 2.0, 4.0, 6.0, 8.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0, 50.0, 60.0, 70.0, 80.0, 90.0, 100.0};
	vector<double> K = {0.1, 0.5, 1.0, 1.5, 2.0, 3.0, 4.0, 6.0, 10.0, 15.0, 50.0, 100.0};


	vector<thread> threads;
	int n_threads = (sweeping == "K") ? K.size() : beta.size();
	for (int i = 0; i < n_threads; i++)
	{
		if (sweeping == "K") {
			thread t(run_angle_eta_simulation, n_steps, beta[2], K[i], system_filename, output_folder, sampling);
			threads.push_back(std::move(t));
		} else if (sweeping == "beta") {
			thread t(run_angle_eta_simulation, n_steps, beta[i], K[2], system_filename, output_folder, sampling);
			threads.push_back(std::move(t));
		} else {
			for (int j = 0; j < K.size(); j++) {
				thread t(run_angle_eta_simulation, n_steps, beta[i], K[j], system_filename, output_folder, sampling);
				threads.push_back(std::move(t));
			}	
		}
		
	}

	for (auto& th : threads) {
		if (th.joinable()) {
			th.join();
		}
	}
}

// void sweep(int n_steps, string system_filename, string output_folder) {
// 	// vector<double> K = {0.1, 0.2, 0.3, 0.4, 0.5, 0.52, 0.54, 0.56, 0.58, 0.6, 0.62, 0.64, 0.66, 0.68, 0.7, 0.8, 0.9, 1.0};
// 	// vector<double> K = {0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0};
// 	vector<double> K = {0.9};
// 	for (int i = 0; i < K.size() ; i++)
// 	{
// 		run_angle_eta_simulation(n_steps, K[i], system_filename, output_folder);
// 	}
// }

int main(int argc, char const *argv[])
{
	int n_steps = atoi(argv[1]);
	string system_filename = argv[2];
	string output_folder = argv[3];
	string sampling = argv[4]; // interval = interval steps, global = global steps in eta
	string sweeping = argv[5]; // K = K sweeping, beta = beta sweeping, both = 2d sweep in K and beta

	threaded_sweep(n_steps, system_filename, output_folder, sampling, sweeping);
	// sweep(n_steps, system_filename, output_folder);
}
