import numpy as np
from matplotlib import pyplot as plt

beta = ["0.100000", "0.500000", "1.000000", "2.000000", "5.000000", "10.000000", "20.000000", "50.000000", "100.000000", "1000.000000"]
beta_float = [0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0, 1000.0]

n_runs = 1
n_steps_per_run = 3000000

def evolution(folder, beta, K):
## Plot evolution of the first 5000 steps of a Monte Carlo run 
	file = np.genfromtxt("./data/" + folder + "/single_systembeta=" + beta + "K="+ K + "j="+str(3)+".dat", usecols=(0,2), skip_footer=n_steps_per_run-10000)

	ax = plt.subplot(121)
	ax.tick_params(direction="in", right=True, top=True)
	plt.plot(file[:5000,0])
	plt.ylabel(r"$\Psi$", fontsize=20)
	plt.xlabel("Monte Carlo time", fontsize=20)

	ax = plt.subplot(122)
	ax.tick_params(direction="in", right=True, top=True)
	plt.plot(file[:5000,1])
	plt.ylabel(r"$\delta_2$", fontsize=20)
	plt.xlabel("Monte Carlo time", fontsize=20)
	plt.show()

def angle_distributions(folder, beta, K):
## Plot distributions of delta_2 for different number of steps
	average = np.empty(n_runs*n_steps_per_run)

	for j in range(n_runs):
		file = np.genfromtxt("./data/" + folder + "/single_systembeta=" + beta + "K="+ K + "j="+str(j)+".dat", usecols=(2))
		average[j*n_steps_per_run:(j+1)*n_steps_per_run] = file

	n_samples = [10000, 200000, 500000, 1500000, 2000000, 3000000]
	axes = []
	boxproperties = dict(boxstyle="square", facecolor="white", alpha=0.5)
	for i in range(len(n_samples)):
		ax = plt.subplot(3,2,i+1)
		axes.append(ax)
		ax.tick_params(direction="in", right=True, top=True, labelbottom=False)
		ax.text(0.98, 0.95, r"$<\delta_2>$ = %.4f" % np.mean(average[:n_samples[i]]), transform=ax.transAxes, fontsize=14, verticalalignment="top", horizontalalignment="right", bbox=boxproperties)
		plt.title(str(n_samples[i]) + " steps", fontsize=20)
		weights = np.ones_like(average[:n_samples[i]])/float(len(average[:n_samples[i]]))
		plt.hist(average[:n_samples[i]], 150, weights=weights, range=(-0.3, 0.3))

	axes[0].set_ylabel("PDF", fontsize=20)
	axes[2].set_ylabel("PDF", fontsize=20)
	axes[4].set_ylabel("PDF", fontsize=20)
	axes[4].set_xlabel(r"$\delta_2$", fontsize=20)
	axes[4].tick_params(labelbottom=True)
	axes[5].set_xlabel(r"$\delta_2$", fontsize=20)
	axes[5].tick_params(labelbottom=True)

	plt.tight_layout()
	plt.show()

def power_distributions(folder, beta, K):
## Plot distributions of power flow at node 2
	average = np.empty(n_runs*n_steps_per_run)
	
	for j in range(n_runs):
		file = np.genfromtxt("./data/" + folder + "/single_systembeta=" + beta + "K=" + K + "j="+str(j)+".dat", usecols=(2))

		file = np.sin(file)
		file = np.multiply(file, -242)
		average[j*n_steps_per_run:(j+1)*n_steps_per_run] = file

		print(np.mean(file))

	# n_samples = [10000, 50000, 200000, 500000, 1500000, 2000000]
	n_samples = [10000, 200000, 500000, 1500000, 2500000, 3000000]
	axes = []
	boxproperties = dict(boxstyle="square", facecolor="white", alpha=0.5)
	for i in range(len(n_samples)):
		ax = plt.subplot(3,2,i+1)
		axes.append(ax)
		ax.tick_params(direction="in", right=True, top=True, labelbottom=False)
		ax.text(0.98, 0.95, r"$<P_2>$ = %.2f" % np.mean(average[:n_samples[i]]), transform=ax.transAxes, fontsize=14, verticalalignment="top", horizontalalignment="right", bbox=boxproperties)
		plt.title(str(n_samples[i]) + " steps", fontsize=20)
		weights = np.ones_like(average[:n_samples[i]])/float(len(average[:n_samples[i]]))
		plt.hist(average[:n_samples[i]], 150, weights=weights, range=(-50, 50))
		
	axes[0].set_ylabel("PDF", fontsize=20)
	axes[2].set_ylabel("PDF", fontsize=20)
	axes[4].set_ylabel("PDF", fontsize=20)
	axes[4].set_xlabel(r"$P_2$", fontsize=20)
	axes[4].tick_params(labelbottom=True)
	axes[5].set_xlabel(r"$P_2$", fontsize=20)
	axes[5].tick_params(labelbottom=True)

	plt.tight_layout()
	plt.show()

def beta_power_distributions(folder):
## Plot distributions of power flow at node 2 for varying beta
	average = np.empty((n_runs*n_steps_per_run, len(beta)))
	
	for i in range(len(beta)):
		for j in range(n_runs):
			print(i,j)
			file = np.genfromtxt("./data/" + folder + "/single_systembeta=" + beta[i] + "K=1.000000" + "j="+str(j)+".dat", usecols=(2))

			file = np.sin(file)
			file = np.multiply(file, -242)
			average[j*n_steps_per_run:(j+1)*n_steps_per_run, i] = file

			print(np.mean(file))

	ranges = [150, 70, 50, 35, 30, 20, 15, 15, 10, 5]
	axes = []
	boxproperties = dict(boxstyle="square", facecolor="white", alpha=0.5)
	for i in range(len(beta)):
		ax = plt.subplot(5,2,i+1)
		axes.append(ax)
		ax.tick_params(direction="in", right=True, top=True, labelbottom=False)
		ax.text(0.98, 0.91, r"$<P_2>$ = %.2f" % np.mean(average[:, i]), transform=ax.transAxes, fontsize=14, verticalalignment="top", horizontalalignment="right", bbox=boxproperties)
		plt.title(r"$\beta = $" + str(beta_float[i]))
		weights = np.ones_like(average[:, i])/float(len(average[:, i]))
		plt.hist(average[:, i], bins=150, weights=weights, range=(-ranges[i]+1, ranges[i]+1))
		plt.xlim(-155,155)
		
	axes[0].set_ylabel("PDF", fontsize=20)
	axes[2].set_ylabel("PDF", fontsize=20)
	axes[4].set_ylabel("PDF", fontsize=20)
	axes[6].set_ylabel("PDF", fontsize=20)
	axes[8].set_ylabel("PDF", fontsize=20)
	axes[8].set_xlabel(r"$P_2$", fontsize=20)
	axes[8].tick_params(labelbottom=True)
	axes[9].set_xlabel(r"$P_2$", fontsize=20)
	axes[9].tick_params(labelbottom=True)

	plt.tight_layout()
	plt.show()

## Example usages
# evolution("120919-singlesystembetaglobalsweep", "1.000000", "1.000000")
# angle_distributions("120919-singlesystembetaglobalsweep", "1.000000", "1.000000")
# power_distributions("120919-singlesystembetaglobalsweep", "1.000000", "1.000000")
beta_power_distributions("120919-singlesystembetaglobalsweep")